@@ -188,6 +188,54 @@ class shellPython {
             throw std::runtime_error("PostPen data not generated");
         }
     }
+
+    pybind11::array_t<double> getTrajectory(const std::size_t target) {
+        if (s.completedTrajectory) {
+            if (target < s.impactSize) {
+                const auto acquired_targets = s.get_trajectory(target);
+                const std::vector<double> &a_x = std::get<0>(acquired_targets);
+                const std::vector<double> &a_y = std::get<1>(acquired_targets);
+                const std::vector<double> &a_y_c =
+                    std::get<2>(acquired_targets);
+
+                const std::size_t target_size = a_x.size();
+
+                /*std::vector<double> temporary_buffer(target_size * 3);
+                assert(a_x.size() == a_y.size() && a_y.size() == a_y_c.size());
+                std::copy(a_x.begin(), a_x.end(), temporary_buffer.begin());
+                std::copy(a_y.begin(), a_y.end(),
+                          temporary_buffer.begin() + target_size);
+                std::copy(a_y_c.begin(), a_y_c.end(),
+                          temporary_buffer.begin() + target_size * 2);*/
+
+                constexpr std::size_t sT = sizeof(double);
+                std::array<size_t, 2> shape = {3, target_size},
+                                      stride = {target_size * sT, sT};
+
+                auto result = pybind11::array_t<double>(shape, stride, nullptr);
+
+                /*pybind11::array_t<double>(pybind11::buffer_info(
+                    nullptr, sT, pybind11::format_descriptor<double>::value, 2,
+                    shape, stride));*/
+
+                pybind11::buffer_info res_buffer_info = result.request();
+                double *res_buffer_ptr =
+                    static_cast<double *>(res_buffer_info.ptr);
+                std::copy(a_x.begin(), a_x.end(), res_buffer_ptr);
+                std::copy(a_y.begin(), a_y.end(), res_buffer_ptr + target_size);
+                std::copy(a_y_c.begin(), a_y_c.end(),
+                          res_buffer_ptr + target_size * 2);
+
+                return result;
+            } else {
+                throw std::runtime_error(
+                    "Invalid target, must be between 0 and impact size - 1 "
+                    "inclusive.");
+            }
+        } else {
+            throw std::runtime_error("Trajectory data not generated");
+        }
+    }
 };
 
 std::string generateShellPythonHash(const shellPython &s) {
@@ -209,8 +257,8 @@ class shellCalcPython : public shellCalc {
     void setDtf(const double dtf) { calc.set_dtf(dtf); }*/
 
     template <numerical Numerical>
-    void calcImpact(shellPython &sp) {
-        calculateImpact<false, Numerical, false>(sp.s);
+    void calcImpact(shellPython &sp, bool addTraj = false) {
+        calculateImpact<Numerical>(sp.s, addTraj);
     }
 
     void calcAngles(shellPython &sp, const double thickness,
@@ -218,7 +266,11 @@ class shellCalcPython : public shellCalc {
         calculateAngles(thickness, inclination, sp.s);
     }
 
-    void calcDispersion(shellPython &sp) { calculateDispersion(sp.s); }
+    void calcDispersion(shellPython &sp, const std::size_t verticalType_i) {
+        dispersion::verticalTypes verticalType =
+            static_cast<dispersion::verticalTypes>(verticalType_i);
+        calculateDispersion(verticalType, sp.s);
+    }
 
     void calcPostPen(shellPython &sp, const double thickness,
                      const double inclination, std::vector<double> angles,
@@ -380,6 +432,7 @@ PYBIND11_MODULE(wows_shell, m) {
              pybind11::arg("owned") = true)
         .def("getPostPen", &shellPython::getPostPen,
              pybind11::arg("owned") = true)
+        .def("getTrajectory", &shellPython::getTrajectory)
         .def("printImpact", &shellPython::printImpact)
         .def("printAngles", &shellPython::printAngles)
         .def("printDispersion", &shellPython::printDispersion)
@@ -398,13 +451,17 @@ PYBIND11_MODULE(wows_shell, m) {
         .def("setYf0", &shellCalcPython::set_yf0)
         .def("setDtf", &shellCalcPython::set_dtf)
         .def("calcImpactForwardEuler",
-             &shellCalcPython::calcImpact<numerical::forwardEuler>)
+             &shellCalcPython::calcImpact<numerical::forwardEuler>,
+             pybind11::arg("shell"), pybind11::arg("addTraj") = false)
         .def("calcImpactAdamsBashforth5",
-             &shellCalcPython::calcImpact<numerical::adamsBashforth5>)
+             &shellCalcPython::calcImpact<numerical::adamsBashforth5>,
+             pybind11::arg("shell"), pybind11::arg("addTraj") = false)
         .def("calcImpactRungeKutta2",
-             &shellCalcPython::calcImpact<numerical::rungeKutta2>)
+             &shellCalcPython::calcImpact<numerical::rungeKutta2>,
+             pybind11::arg("shell"), pybind11::arg("addTraj") = false)
         .def("calcImpactRungeKutta4",
-             &shellCalcPython::calcImpact<numerical::rungeKutta4>)
+             &shellCalcPython::calcImpact<numerical::rungeKutta4>,
+             pybind11::arg("shell"), pybind11::arg("addTraj") = false)
         .def("calcAngles", &shellCalcPython::calcAngles)
         .def("calcDispersion", &shellCalcPython::calcDispersion)
         .def("calcPostPen", &shellCalcPython::calcPostPen);
@@ -450,6 +507,26 @@ PYBIND11_MODULE(wows_shell, m) {
         .value("fuseDegrees", angle::angleIndices::fuseDegrees)
         .export_values();
 
+    pybind11::enum_<dispersion::dispersionIndices>(m, "dispersionIndices",
+                                                   pybind11::arithmetic())
+        .value("maxHorizontal", dispersion::dispersionIndices::maxHorizontal)
+        .value("standardHorizontal",
+               dispersion::dispersionIndices::standardHorizontal)
+        .value("halfHorizontal", dispersion::dispersionIndices::halfHorizontal)
+        .value("maxVertical", dispersion::dispersionIndices::maxVertical)
+        .value("standardVertical",
+               dispersion::dispersionIndices::standardVertical)
+        .value("halfVertical", dispersion::dispersionIndices::halfVertical)
+        .value("maxArea", dispersion::dispersionIndices::maxArea)
+        .value("standardArea", dispersion::dispersionIndices::standardArea)
+        .value("halfArea", dispersion::dispersionIndices::halfArea);
+
+    pybind11::enum_<dispersion::verticalTypes>(m, "verticalTypes",
+                                               pybind11::arithmetic())
+        .value("horizontal", dispersion::verticalTypes::horizontal)
+        .value("normal", dispersion::verticalTypes::normal)
+        .value("vertical", dispersion::verticalTypes::vertical);
+
     pybind11::enum_<post::postPenIndices>(m, "postPenIndices",
                                           pybind11::arithmetic())
         .value("angle", post::postPenIndices::angle)
