@@ -18,9 +18,12 @@
 #include "controlEnums.hpp"
 #include "shell.hpp"
 #include "utility.hpp"
+
+#if defined(__SSE4_1__) || defined(__AVX__)
 #include "version2/vectorclass.h"
 #include "version2/vectormath_exp.h"
 #include "version2/vectormath_trig.h"
+#endif
 
 namespace wows_shell {
 class shellCalc {
@@ -203,10 +206,14 @@ class shellCalc {
         if constexpr (AddTraj) {
             for (uint32_t i = 0, j = start; i < vSize; ++i, ++j) {
                 if (j < s.impactSize) {
-                    s.trajectories[2 * (j)].clear();
-                    s.trajectories[2 * (j) + 1].clear();
-                    s.trajectories[2 * (j)].push_back(x0);
-                    s.trajectories[2 * (j) + 1].push_back(y0);
+                    s.trajectories[s.trajectories_width * (j)].clear();
+                    s.trajectories[s.trajectories_width * (j) + 1].clear();
+                    s.trajectories[s.trajectories_width * (j) + 2].clear();
+                    s.trajectories[s.trajectories_width * (j)].push_back(x0);
+                    s.trajectories[s.trajectories_width * (j) + 1].push_back(
+                        y0);
+                    s.trajectories[s.trajectories_width * (j) + 2].push_back(
+                        utility::compress_height(y0));
                 }
             }
         }
@@ -248,13 +255,37 @@ class shellCalc {
             const uint32_t loopSize =
                 std::min<uint32_t>(vSize, s.impactSize - start);
             for (uint32_t i = 0, j = start; i < loopSize; ++i, ++j) {
+                double x, y;
 #if defined(__SSE4_1__) || defined(__AVX__)
-                s.trajectories[2 * (j)].push_back(xR[i]);
-                s.trajectories[2 * (j) + 1].push_back(yR[i]);
+                x = xR[i], y = yR[i];
 #else
-                s.trajectories[2 * (j)].push_back(xy[i]);
-                s.trajectories[2 * (j) + 1].push_back(xy[i + vSize]);
+                x = xy[i], y = xy[i + vSize];
 #endif
+                std::vector<double> &target_x =
+                    s.trajectories[s.trajectories_width * (j)];
+                std::vector<double> &target_y =
+                    s.trajectories[s.trajectories_width * (j) + 1];
+                std::vector<double> &target_y_c =
+                    s.trajectories[s.trajectories_width * (j) + 2];
+                std::size_t current_traj_length = target_x.size();
+                // All targeted trajectory vectors should have the same length
+
+                const auto add_point = [&]() {
+                    target_x.push_back(x);
+                    target_y.push_back(y);
+                    target_y_c.push_back(utility::compress_height(y));
+                };
+
+                constexpr bool trim_duplicates = true;
+                if constexpr (trim_duplicates) {
+                    if (current_traj_length < 2 ||
+                        (target_x[current_traj_length - 1] != x &&
+                         target_y[current_traj_length - 1] != y)) {
+                        add_point();
+                    }
+                } else {
+                    add_point();
+                }
             }
         };
 
@@ -839,29 +870,29 @@ class shellCalc {
         return vSize - (processedSize % vSize) + processedSize;
     }
     // Templates to reduce branching
-    template <auto Numerical, bool Hybrid>
+    template <auto Numerical>
     void calculateImpact(
         shell &s, bool addTraj,
         std::size_t nThreads = std::thread::hardware_concurrency()) const {
         if (addTraj) {
-            calculateImpact<true, Numerical, Hybrid>(s, nThreads);
+            calculateImpact<true, Numerical>(s, nThreads);
         } else {
-            calculateImpact<false, Numerical, Hybrid>(s, nThreads);
+            calculateImpact<false, Numerical>(s, nThreads);
         }
     }
 
-    template <bool AddTraj, auto Numerical, bool Hybrid>
+    template <bool AddTraj, auto Numerical>
     void calculateImpact(
         shell &s,
         std::size_t nThreads = std::thread::hardware_concurrency()) const {
         if (s.enableNonAP) {
-            calculateImpact<true, Numerical, Hybrid, true>(s, nThreads);
+            calculateImpact<AddTraj, Numerical, true>(s, nThreads);
         } else {
-            calculateImpact<true, Numerical, Hybrid, false>(s, nThreads);
+            calculateImpact<AddTraj, Numerical, false>(s, nThreads);
         }
     }
 
-    template <bool AddTraj, auto Numerical, bool Hybrid, bool nonAP>
+    template <bool AddTraj, auto Numerical, bool nonAP>
     void calculateImpact(
         shell &s,
         std::size_t nThreads = std::thread::hardware_concurrency()) const {
@@ -869,7 +900,7 @@ class shellCalc {
             static_cast<std::size_t>(maxA / precision - minA / precision) + 1;
         s.impactSizeAligned = calculateAlignmentSize(s.impactSize);
         if constexpr (AddTraj) {
-            s.trajectories.resize(2 * s.impactSize);
+            s.trajectories.resize(s.trajectories_width * s.impactSize);
         }
         s.impactData.resize(impact::maxColumns * s.impactSizeAligned);
 
@@ -884,6 +915,11 @@ class shellCalc {
             });
 
         s.completedImpact = true;
+        if constexpr (AddTraj) {
+            s.completedTrajectory = true;
+        } else {
+            s.completedTrajectory = false;
+        }
     }
 
     template <auto Numerical>
@@ -1200,30 +1236,44 @@ class shellCalc {
     }
 
     // Dispersion Section
+    void calculateDispersion(
+        const dispersion::verticalTypes verticalType, shell &s,
+        std::size_t nThreads = std::thread::hardware_concurrency()) const {
+        using verticalTypes = dispersion::verticalTypes;
+        if (verticalType == verticalTypes::horizontal) {
+            calculateDispersion<verticalTypes::horizontal>(s);
+        } else if (verticalType == verticalTypes::normal) {
+            calculateDispersion<verticalTypes::normal>(s);
+        } else {
+            calculateDispersion<verticalTypes::vertical>(s);
+        }
+    }
+
+    template <dispersion::verticalTypes verticalType>
     void calculateDispersion(
         shell &s,
         std::size_t nThreads = std::thread::hardware_concurrency()) const {
         checkRunImpact(s);
         s.dispersionData.resize(dispersion::maxColumns * s.impactSizeAligned);
-        if (nThreads > std::thread::hardware_concurrency()) {
-            nThreads = std::thread::hardware_concurrency();
-        }
         std::size_t length = ceil(static_cast<double>(s.impactSize) / vSize);
         std::size_t assigned = assignThreadNum(length, nThreads);
         if (s.zeroDelimSlope >= s.delimMaxSlope) {
-            mtFunctionRunner(
-                assigned, length, s.impactSize,
-                [&](const std::size_t i) { dispersionGroup<true>(i, s); });
+            mtFunctionRunner(assigned, length, s.impactSize,
+                             [&](const std::size_t i) {
+                                 dispersionGroup<true, verticalType>(i, s);
+                             });
         } else {
-            mtFunctionRunner(
-                assigned, length, s.impactSize,
-                [&](const std::size_t i) { dispersionGroup<false>(i, s); });
+            mtFunctionRunner(assigned, length, s.impactSize,
+                             [&](const std::size_t i) {
+                                 dispersionGroup<false, verticalType>(i, s);
+                             });
         }
         s.completedDispersion = true;
     }
 
-    template <bool convex>
+    template <bool convex, dispersion::verticalTypes verticalType>
     void dispersionGroup(const std::size_t startIndex, shell &s) const {
+        using verticalTypes = dispersion::verticalTypes;
 #if defined(__SSE4_2__) || defined(__AVX__)
         const std::size_t i = startIndex;
         const VT distance =
@@ -1252,8 +1302,16 @@ class shellCalc {
         const VT verticalRatio = min(verticalRatioUncapped, VT(s.maxRadius));
         // Results will never be higher than s.maxRadius
 
-        const VT vertical =
-            horizontal * verticalRatio / sin(impactAngle * VT(-1));
+        const VT vertical = [&]() -> VT {
+            const VT verticalNormal = horizontal * verticalRatio;
+            if constexpr (verticalType == verticalTypes::horizontal) {
+                return verticalNormal / sin(impactAngle * VT(-1));
+            } else if constexpr (verticalType == verticalTypes::normal) {
+                return verticalNormal;
+            } else {
+                return verticalNormal / cos(impactAngle * VT(-1));
+            }
+        }();
         /*std::cout << verticalRatioUncapped[0] << " " <<
            verticalRatioUncapped[1]
                   << " " << verticalRatioUncapped[2] << " "
@@ -1317,8 +1375,16 @@ class shellCalc {
             // std::cout << distance << " " << verticalRatio << "\n";
             // Results will never be higher than s.maxRadius
 
-            const double vertical =
-                horizontal * verticalRatio / sin(impactAngle * -1);
+            const double vertical = [&]() -> double {
+                const double verticalNormal = horizontal * verticalRatio;
+                if constexpr (verticalType == verticalTypes::horizontal) {
+                    return verticalNormal / sin(impactAngle * -1);
+                } else if constexpr (verticalType == verticalTypes::normal) {
+                    return verticalNormal;
+                } else {
+                    return verticalNormal / cos(impactAngle * -1);
+                }
+            }();
             const double area = M_PI * horizontal * vertical;
 
             s.get_dispersion(i, dispersion::dispersionIndices::maxHorizontal) =
